/*
 * Boltzmann 3D, a kinetic theory demonstrator
 * Copyright (C) 2013 Dr. Randall B. Shirts
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.byu.chem.boltzmann.fullapplication.view.simulationsettings.components;

import edu.byu.chem.boltzmann.modules.sharedcomponents.dimensionselector.ButtonGroupWithValues;
import edu.byu.chem.boltzmann.modules.sharedcomponents.dimensionselector.ButtonGroupWithValues.SelectionChangedListener;
import edu.byu.chem.boltzmann.modules.sharedcomponents.dimensionselector.DimensionSelector;
import edu.byu.chem.boltzmann.utils.data.SimulationInfo.Dimension;
import edu.byu.chem.boltzmann.view.utils.events.BoltzmannListenerHandler;

/**
 *
 * @author Derek Manwaring
 * 14 Dec 2011
 */
public class FullDimensionSelector extends javax.swing.JPanel implements DimensionSelector {
    
    BoltzmannListenerHandler<DimensionChangedListener, DimensionChangedEvent> listeners = 
            new BoltzmannListenerHandler<DimensionChangedListener, DimensionChangedEvent>();
                
    ButtonGroupWithValues<Dimension> btnGrpDimension = 
            new ButtonGroupWithValues<Dimension>(
                    new SelectionChangedListener<Dimension>() {
                        public void selectionChanged(Dimension newSelectedDimension) {
                            listeners.notifyListeners(new DimensionChangedEvent(newSelectedDimension));
                        }                      
                    });
    
    private static final Dimension DEFAULT_DIMENSION = Dimension.TWO;
    
    /** Creates new form FullDimensionSelector */
    public FullDimensionSelector() {
        initComponents();
        
        btnGrpDimension.addButtonWithValue(radBtn1Dimensional, Dimension.ONE);
        btnGrpDimension.addButtonWithValue(radBtn2Dimensional, Dimension.TWO);
        btnGrpDimension.addButtonWithValue(radBtn3Dimensional, Dimension.THREE);      
        
        btnGrpDimension.setSelectedValue(DEFAULT_DIMENSION);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        radBtn3Dimensional = new javax.swing.JRadioButton();
        radBtn2Dimensional = new javax.swing.JRadioButton();
        radBtn1Dimensional = new javax.swing.JRadioButton();

        setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.PAGE_AXIS));

        radBtn3Dimensional.setText("3 Dimensional");
        add(radBtn3Dimensional);

        radBtn2Dimensional.setSelected(true);
        radBtn2Dimensional.setText("2 Dimensional");
        add(radBtn2Dimensional);

        radBtn1Dimensional.setText("1 Dimensional");
        add(radBtn1Dimensional);
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    javax.swing.JRadioButton radBtn1Dimensional;
    javax.swing.JRadioButton radBtn2Dimensional;
    javax.swing.JRadioButton radBtn3Dimensional;
    // End of variables declaration//GEN-END:variables

    public void addListener(DimensionChangedListener listener) {
        listeners.addListener(listener);
    }

    public Dimension getSelectedDimension() {
        return btnGrpDimension.getSelectedValue();
    }

    public void setSelectedDimension(Dimension selectedDimension) {
        btnGrpDimension.setSelectedValue(selectedDimension);
    }
}
